﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Windows.Printing;
using C1.Silverlight.DataGrid;
using C1.Silverlight.DataGrid.PrintPreview;
using C1.Silverlight.Pdf;

namespace C1DataGrid_Printing
{
    public partial class MainPage : UserControl
    {
        ICollectionView _data = Product.GetProducts(100);

        public MainPage()
        {
            InitializeComponent();

            // populate grid
            grid.ItemsSource = _data;
        }

        #region ** event handlers

        // toggle grouping
        void _chkGroup_Click(object sender, RoutedEventArgs e)
        {
            using (_data.DeferRefresh())
            {
                var gd = _data.GroupDescriptions;
                gd.Clear();
                if (_chkGroup.IsChecked.Value)
                {
                    _data.GroupDescriptions.Add(new System.Windows.Data.PropertyGroupDescription("Line"));
                    _data.GroupDescriptions.Add(new System.Windows.Data.PropertyGroupDescription("Color"));
                }
            }
        }

        // toggle merging
        void _chkMerge_Click(object sender, RoutedEventArgs e)
        {
            //grid.AllowMerging = _chkMerge.IsChecked.Value ? AllowMerging.Cells : AllowMerging.None;
        }

        // toggle freezing
        void _chkFreeze_Click(object sender, RoutedEventArgs e)
        {
            if (_chkFreeze.IsChecked.Value)
            {
                grid.CanUserFreezeColumns = DataGridColumnFreezing.Left;
            }
            else
            {
                grid.CanUserFreezeColumns = DataGridColumnFreezing.None;
            }
        }
        #endregion

        #region ** printing

        // basic printing
        void _btnBasicPrinting_Click(object sender, RoutedEventArgs e)
        {
            // get scale mode and margin
            var scaleMode =
                _cmbZoom.SelectedIndex == 0 ? ScaleMode.ActualSize :
                _cmbZoom.SelectedIndex == 1 ? ScaleMode.PageWidth :
                ScaleMode.SinglePage;
            var margin =
                _cmbMargins.SelectedIndex == 0 ? 0 :
                _cmbMargins.SelectedIndex == 1 ? 96.0 / 2 :
                96.0;

            // print the grid
            grid.Print("C1DataGrid", scaleMode, new Thickness(margin), 20);
        }

        // advanced printing
        void _btnAdvancedPrinting_Click(object sender, RoutedEventArgs e)
        {
            var pd = new System.Windows.Printing.PrintDocument();
            pd.BeginPrint += pd_BeginPrint;
            pd.PrintPage += pd_PrintPage;
            pd.EndPrint += pd_EndPrint;
            pd.Print("C1DataGrid");
        }

        List<FrameworkElement> _pages;
        double _margin;
        ScaleMode _scaleMode;
        int _currentPage;
        void pd_BeginPrint(object sender, BeginPrintEventArgs e)
        {
            _pages = null;

            // get margins, scale mode
            _margin =
                _cmbMargins.SelectedIndex == 0 ? 0 :
                _cmbMargins.SelectedIndex == 1 ? 96.0 / 2 :
                96.0;
            _scaleMode =
                _cmbZoom.SelectedIndex == 0 ? ScaleMode.ActualSize :
                _cmbZoom.SelectedIndex == 1 ? ScaleMode.PageWidth :
                ScaleMode.SinglePage;
        }
        void pd_PrintPage(object sender, PrintPageEventArgs e)
        {
            if (_pages == null)
            {
                // calculate page size, discounting margins
                var sz = e.PrintableArea;
                sz.Width -= 2 * _margin;
                sz.Height -= 2 * _margin;

                // generate images for every page
                _currentPage = 0;
                int verticalCount;
                _pages = grid.GetPageImages(_scaleMode, sz, new Thickness(0), 100, out verticalCount);
            }

            // create page element
            var pageTemplate = new PageTemplate();

            // set margins
            pageTemplate.SetPageMargin(new Thickness(_margin));

            // set content
            pageTemplate.PageContent.Child = _pages[_currentPage];
            pageTemplate.PageContent.Stretch = _scaleMode == ScaleMode.ActualSize
                ? System.Windows.Media.Stretch.None
                : System.Windows.Media.Stretch.Uniform;

            // set footer text
            pageTemplate.FooterRight.Text = string.Format("Page {0} of {1}",
                _currentPage + 1, _pages.Count);

            // render this page
            e.PageVisual = pageTemplate;

            // move on to next page
            _currentPage++;
            e.HasMorePages = _currentPage < _pages.Count;
        }
        void pd_EndPrint(object sender, EndPrintEventArgs e)
        {
            _pages = null;
        }

        private void _btnPrintPreview_Click(object sender, RoutedEventArgs e)
        {
            // get scale mode and margin
            var scaleMode =
                _cmbZoom.SelectedIndex == 0 ? ScaleMode.ActualSize :
                _cmbZoom.SelectedIndex == 1 ? ScaleMode.PageWidth :
                ScaleMode.SinglePage;
            var margin =
                _cmbMargins.SelectedIndex == 0 ? 0 :
                _cmbMargins.SelectedIndex == 1 ? 96.0 / 2 :
                96.0;

            // call extension PrintPreview
            grid.PrintPreview();
        }
        #endregion

        private void _btnSavePdf_Click(object sender, RoutedEventArgs e)
        {
            // get scale mode and margin
            var scaleMode =
                _cmbZoom.SelectedIndex == 0 ? ScaleMode.ActualSize :
                _cmbZoom.SelectedIndex == 1 ? ScaleMode.PageWidth :
                ScaleMode.SinglePage;
            var margin =
                _cmbMargins.SelectedIndex == 0 ? 0 :
                _cmbMargins.SelectedIndex == 1 ? 96.0 / 2 :
                96.0;

            var dialog = new System.Windows.Controls.SaveFileDialog();

            dialog.DefaultExt = "*.pdf";
            dialog.Filter = "Portable document format (*.pdf)|*.pdf|All files (*.*)|*.*";

            if (dialog.ShowDialog() == false) return;

            using (var s = dialog.OpenFile())
            {
                var pdfDoc = new C1PdfDocument();
                pdfDoc.DocumentInfo.Title = "Sample DataGrid";
                //pdfDoc.PageSize = new Size(100, 100);
                //pdfDoc.PageRectangle = new Rect(10, 10, 80, 80);
                pdfDoc = grid.ToPdf(new PageSettings() { ScaleMode = scaleMode, Margins = new Thickness(margin), PaperKind= PaperKind.Letter });
                pdfDoc.Save(s);
            }
        }
    }
}
